import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;



public class Thesaurus {
	
	private String thesaurusFile;
	private HashMap<String, String> thesWordMap = new HashMap<String, String>(); 
	
	
	/*
	 * the map goes < wordToBeChanged, wordToChangeTo >.
	 */
	public Thesaurus()
	{
		//
	}
	
	public Thesaurus(String fileName)
	{
		this.buildThesaurus(fileName);
	}
	
	public void buildThesaurus (String fileName) 
	{
		thesaurusFile = fileName;

		try {
			BufferedReader in = new BufferedReader(new FileReader(fileName));
			String str;
			while ((str = in.readLine()) != null) {

				// this statement reads the line from the file and print it to
				// the console.
				//System.out.println(dis.readLine());

				String[] finalWord = str.split(":");
				String targetWord;
				if (finalWord[0].trim().equalsIgnoreCase("STOPWORDS"))
					targetWord = " ";
				else 
					targetWord = finalWord[0].toUpperCase().trim();

				String[] words = finalWord[1].split(",");

				for (int i = 0; i<words.length; i++)
				{

					thesWordMap.put(words[i].toUpperCase().trim(), targetWord);
				}
			}
			in.close();
		}
		catch (IOException e) 
		{
			System.err.print("ERROR: Failed to open file " + this.thesaurusFile);
			System.exit(-1);
		}
	}

	public void printThesaurus (){
		Object[] keys = thesWordMap.keySet().toArray();
		for (int i = 0; i < keys.length; i++)
		{
			//thesaurus.get(keys[i]);
			System.out.println(keys[i] + " : " + thesWordMap.get(keys[i]));
		}
	}
	
	public String getThesaurus()
	{
		StringBuffer output = new StringBuffer();
		Object[] keys = thesWordMap.keySet().toArray();
		for (int i = 0; i < keys.length; i++)
		{
			// key = word that need to be changed
			output.append(String.format("%s:%s\n",keys[i], thesWordMap.get(keys[i])));
		}
		
		return output.toString();
	}
	
	public String getThesaurusLine()
	{
		StringBuffer output = new StringBuffer();
		Object[] keys = thesWordMap.keySet().toArray();
		for (int i = 0; i < keys.length; i++)
			output.append(String.format("%s=%s ",keys[i], thesWordMap.get(keys[i])));
		
		output.append("\n");
		return output.toString();
	}

	public String applyToString(String st){
		/* this might need to be optimized.  I am doing a quick replaceAll but 
		 * It might be slow because it iterates through the thesaurus. It would
		 * probably be easier to iterate through the input String str by converting
		 *  it to an array, but then phrases would be a problem
		 * .  Some Ideas to speed things up are to mark the phrases and to a reg-
		 * exp search on those and the rest to an array search
		 * 
		 */
		// make everything upper case
		st = st.toUpperCase();
		StringBuffer str = new StringBuffer();
		str.append(str);

		// replace all periods, commas, semicolons, colons to spaces
		Pattern p = Pattern.compile("[\\\\.,_;:]");
		Matcher m = p.matcher(st);
		st = m.replaceAll(" ");
//		st.replaceAll("I", " ");
		//System.out.println(str);
// loop through the thesaurus an replace
		Iterator<String> thesIterator = thesWordMap.keySet().iterator();
		while (thesIterator.hasNext())
		{
			Object key = thesIterator.next();
			// this will include plurals (will match both CAT and CATS)
			//st.replaceAll((String)key+"S?", thesWordMap.get(key));
			p = Pattern.compile(("\\b" + (String)key+"S?\\b"));
			m = p.matcher(st);
			st = m.replaceAll(thesWordMap.get(key));
			//str = str.replaceAll((String)key, thesWordMap.get(key));
		}
		return st.toString();
	}
	
	public String fileName()
	{
		return thesaurusFile;
	}
	
	public void load(String thesString)
	{
		String[] thesElements = thesString.split("\n");
		for(int i = 0; i < thesElements.length; i++)
		{
			String[] a = thesElements[i].split(":");
			thesWordMap.put(a[0], a[1]);
		}
	}

}
